@@ -4,7 +4,6 @@ sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
 	my $self = {
-			prolog_comments         => [],
 			xml_decl                => undef,
 			doctype_decl            => undef,
 			root_name               => "",
@@ -24,13 +23,12 @@ use strict;
 
 use vars qw($VERSION);
 
-$VERSION="0.10";
+$VERSION="0.11";
 
 sub new {
 	my $proto = shift;
 	my $class = ref($proto) || $proto;
 	my $self = {
-			prolog      => 1,
 			doc         => new XML::Handler::Dtd2Html::Document(),
 			comments    => []
 	};
@@ -52,17 +50,12 @@ sub end_document {
 
 sub comment {
 	my $self = shift;
-	if ($self->{prolog}) {
-		push @{$self->{doc}->{prolog_comments}}, shift;
-	} else {
-		push @{$self->{comments}}, shift;
-	}
+	push @{$self->{comments}}, shift;
 }
 
 sub notation_decl {
 	my $self = shift;
 	my ($decl) = @_;
-	$self->{prolog} = 0;
 	if (scalar @{$self->{comments}}) {
 		$decl->{comments} = [@{$self->{comments}}];
 		$self->{comments} = [];
@@ -76,7 +69,6 @@ sub notation_decl {
 sub unparsed_entity_decl {
 	my $self = shift;
 	my ($decl) = @_;
-	$self->{prolog} = 0;
 	if (scalar @{$self->{comments}}) {
 		$decl->{comments} = [@{$self->{comments}}];
 		$self->{comments} = [];
@@ -90,7 +82,6 @@ sub unparsed_entity_decl {
 sub entity_decl {
 	my $self = shift;
 	my ($decl) = @_;
-	$self->{prolog} = 0;
 	if (scalar @{$self->{comments}}) {
 		$decl->{comments} = [@{$self->{comments}}];
 		$self->{comments} = [];
@@ -106,13 +97,12 @@ sub entity_decl {
 sub element_decl {
 	my $self = shift;
 	my ($decl) = @_;
-	$self->{prolog} = 0;
 	if (scalar @{$self->{comments}}) {
 		$decl->{comments} = [@{$self->{comments}}];
 		$self->{comments} = [];
 	}
 	$decl->{type} = "element";
-	$decl->{used_by} = [];
+	$decl->{used_by} = {};
 	$decl->{uses} = [];
 	my $name = $decl->{Name};
 	$self->{doc}->{hash_element}->{$name} = $decl;
@@ -135,6 +125,10 @@ sub attlist_decl {
 sub doctype_decl {
 	my $self = shift;
 	my ($decl) = @_;
+	if (scalar @{$self->{comments}}) {
+		$decl->{comments} = [@{$self->{comments}}];
+		$self->{comments} = [];
+	}
 	$self->{doc}->{doctype_decl} = $decl;
 	$self->{doc}->{root_name} = $decl->{Name};
 }
@@ -148,9 +142,9 @@ package XML::Handler::Dtd2Html::Document;
 
 sub _cross_ref {
 	my $self = shift;
+	my($flag_zombi) = @_;
 
-	foreach my $decl (values %{$self->{hash_element}}) {
-		my $name = $decl->{Name};
+	while (my($name, $decl) = each %{$self->{hash_element}}) {
 		my $model = $decl->{Model};
 		while ($model) {
 			for ($model) {
@@ -166,39 +160,76 @@ sub _cross_ref {
 						and last;
 				s/^([A-Za-z_:][0-9A-Za-z\.\-_:]*)//
 						and push(@{$self->{hash_element}->{$name}->{uses}}, $1),
-						and push(@{$self->{hash_element}->{$1}->{used_by}}, $name),
+						and $self->{hash_element}->{$1}->{used_by}->{$name} = 1,
 						    last;
 				s/^([\S]+)//
 						and warn __PACKAGE__,":_cross_ref INTERNAL_ERROR $1\n",
 						    last;
+			}
 		}
 	}
+
+	if ($flag_zombi) {
+		my $one_more_time = 1;
+		while ($one_more_time) {
+			$one_more_time = 0;
+			while (my($elt_name, $elt_decl) = each %{$self->{hash_element}}) {
+				next if ($elt_name eq $self->{root_name});
+				unless (scalar keys %{$elt_decl->{used_by}}) {
+					delete $self->{hash_element}->{$elt_name};
+					foreach my $child (@{$elt_decl->{uses}}) {
+						my $decl = $self->{hash_element}->{$child};
+						delete $decl->{used_by}->{$elt_name};
+						$one_more_time = 1;
+					}
+				}
+			}
+		}
 	}
 }
 
 sub _format_head {
 	my $self = shift;
-	my($FH, $title) = @_;
+	my($FH, $title, $frameset) = @_;
+	my $now = localtime();
 	print $FH "<?xml version='1.0' encoding='ISO-8859-1'?>\n";
-	print $FH "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Transitional//EN' 'xhtml1-transitional.dtd'>\n";
+	if ($frameset) {
+		print $FH "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Frameset//EN' 'xhtml1-frameset.dtd'>\n";
+	} else {
+		print $FH "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' 'xhtml1-strict.dtd'>\n";
+	}
 	print $FH "<html xmlns='http://www.w3.org/1999/xhtml'>\n";
 	print $FH "\n";
 	print $FH "  <head>\n";
 	print $FH "    <meta name='generator' content='dtd2html (Perl)' />\n";
+	print $FH "    <meta name='date' content='",$now,"' />\n";
 	print $FH "    <meta http-equiv='Content-Type' content='text/html; charset=ISO-8859-1' />\n";
 	print $FH "    <title>",$title,"</title>\n";
+	unless ($frameset) {
+		print $FH "    <style type='text/css'>\n";
+		print $FH "      h2 {color: red}\n";
+		print $FH "      span.keyword1 {color: teal}\n";
+		print $FH "      span.keyword2 {color: maroon}\n";
+		print $FH "      p.comment {color: green}\n";
+		print $FH "      span.comment {color: green}\n";
+		print $FH "      hr {text-align: center}\n";
+		print $FH "    </style>\n";
+	}
 	print $FH "  </head>\n";
 	print $FH "\n";
-	print $FH "  <body>\n";
 }
 
 sub _format_tail {
 	my $self = shift;
-	my($FH) = @_;
-	print $FH "<hr align='center'/>\n";
-	print $FH "    <i>Generated by dtd2html</i>\n";
-	print $FH "\n";
-	print $FH "  </body>\n";
+	my($FH, $frameset) = @_;
+	if ($frameset) {
+		print $FH "  <noframes>\n";
+		print $FH "    <body>\n";
+		print $FH "      <h1>Sorry!</h1>\n";
+		print $FH "      <h3>This page must be viewed by a browser that is capable of viewing frames.</h3>\n";
+		print $FH "    </body>\n";
+		print $FH "  </noframes>\n";
+	}
 	print $FH "\n";
 	print $FH "</html>\n";
 }
@@ -217,13 +248,13 @@ sub _format_content_model {
 					and $str .= $1,
 					    last;
 			s/^(EMPTY)//
-					and $str .= "<font color='#0000A0'>" . $1 . "</font>",
+					and $str .= "<span class='keyword1'>" . $1 . "</span>",
 					    last;
 			s/^(ANY)//
-					and $str .= "<font color='#0000A0'>" . $1 . "</font>",
+					and $str .= "<span class='keyword1'>" . $1 . "</span>",
 					    last;
 			s/^(#PCDATA)//
-					and $str .= "<font color='#0000A0'>" . $1 . "</font>",
+					and $str .= "<span class='keyword1'>" . $1 . "</span>",
 					    last;
 			s/^([A-Za-z_:][0-9A-Za-z\.\-_:]*)//
 					and $str .= "<a href='#elt_" . $1 . "'>" . $1 . "</a>",
@@ -276,79 +307,99 @@ sub _process_text {
 	return $lead . join('', @words) . $trail;
 }
 
+sub generateAlpha {
+	my $self = shift;
+	my ($FH, $frameset, $outfile) = @_;
+
+	my @elements = sort keys %{$self->{hash_element}};
+	if (scalar @elements) {
+		print $FH "<h2>Element index.</h2>\n";
+		print $FH "<dl>\n";
+		foreach (@elements) {
+			if ($frameset) {
+				print $FH "    <dt><a target='main' href='",$outfile,".main.html#elt_",$_,"'><b>",$_,"</b></a>";
+			} else {
+				print $FH "    <dt><a href='#elt_",$_,"'><b>",$_,"</b></a>";
+			}
+			print $FH " (root)" if ($_ eq $self->{root_name});
+			print $FH "</dt>\n";
+		}
+		print $FH "</dl>\n";
+	}
+	my @entities = sort keys %{$self->{hash_entity}};
+	if (scalar @entities) {
+		print $FH "<h2>General Entity index.</h2>\n";
+		print $FH "<dl>\n";
+		foreach (@entities) {
+			if ($frameset) {
+				print $FH "    <dt><a target='main' href='",$outfile,".main.html#ent_",$_,"'><b>",$_,"</b></a></dt>\n";
+			} else {
+				print $FH "    <dt><a href='#ent_",$_,"'><b>",$_,"</b></a></dt>\n";
+			}
+		}
+		print $FH "</dl>\n";
+	}
+	my @notations = sort keys %{$self->{hash_notation}};
+	if (scalar @notations) {
+		print $FH "<h2>Notation index.</h2>\n";
+		print $FH "<dl>\n";
+		foreach (@notations) {
+			if ($frameset) {
+				print $FH "    <dt><a target='main' href='",$outfile,".main.html#not_",$_,"'><b>",$_,"</b></a></dt>\n";
+			} else {
+				print $FH "    <dt><a href='#not_",$_,"'><b>",$_,"</b></a></dt>\n";
+			}
+		}
+		print $FH "</dl>\n";
+	}
+}
+
 sub _mk_tree {
 	my $self = shift;
-	my ($FH, $name) = @_;
+	my ($FH, $frameset, $outfile, $name) = @_;
 	my %done = ();
 
 	$self->{hash_element}->{$name}->{done} = 1;
 	print $FH "<ul>\n";
-	foreach (sort @{$self->{hash_element}->{$name}->{uses}}) {
+	foreach (@{$self->{hash_element}->{$name}->{uses}}) {
 		next if ($_ eq $name);
 		next if (exists $done{$_});
 		$done{$_} = 1;
-		print $FH "  <li><a href='#elt_",$_,"'><b>",$_,"</b></a>\n";
-		$self->_mk_tree($FH, $_)
+		if ($frameset) {
+			print $FH "  <li><a target='main' href='",$outfile,".main.html#elt_",$_,"'><b>",$_,"</b></a>\n";
+		} else {
+			print $FH "  <li><a href='#elt_",$_,"'><b>",$_,"</b></a>\n";
+		}
+		$self->_mk_tree($FH, $frameset, $outfile, $_)
 				unless (exists $self->{hash_element}->{$_}->{done});
 		print $FH "  </li>\n";
 	}
 	print $FH "</ul>\n";
 }
 
-sub printToFileHandle {
+sub generateTree {
 	my $self = shift;
-	my ($FH) = @_;
+	my ($FH, $frameset, $outfile) = @_;
 
-	$self->_cross_ref();
-	my @notations = sort keys %{$self->{hash_notation}};
-	my @entities = sort keys %{$self->{hash_entity}};
-	my @elements = sort keys %{$self->{hash_element}};
-
-	$self->_format_head($FH, "DTD " . $self->{root_name});
-	print $FH "<h1>DTD ",$self->{root_name},"</h1>\n";
-	print $FH "<hr align='center'/>\n";
-	if (scalar @elements) {
-		print $FH "<h2><font color='#FF0000'>Element index.</font></h2>\n";
-		print $FH "<dl>\n";
-		foreach (@elements) {
-			if ($_ eq $self->{root_name}) {
-				print $FH "    <dt><a href='#elt_",$_,"'><b>",$_,"</b></a> (root)</dt>\n";
-			} else {
-				print $FH "    <dt><a href='#elt_",$_,"'><b>",$_,"</b></a></dt>\n";
-			}
-		}
-		print $FH "</dl>\n";
-	}
-	if (scalar @entities) {
-		print $FH "<h2><font color='#FF0000'>General Entity index.</font></h2>\n";
-		print $FH "<dl>\n";
-		foreach (@entities) {
-			print $FH "    <dt><a href='#ent_",$_,"'><b>",$_,"</b></a></dt>\n";
-		}
-		print $FH "</dl>\n";
-	}
-	if (scalar @notations) {
-		print $FH "<h2><font color='#FF0000'>Notation index.</font></h2>\n";
-		print $FH "<dl>\n";
-		foreach (@notations) {
-			print $FH "    <dt><a href='#not_",$_,"'><b>",$_,"</b></a></dt>\n";
-		}
-		print $FH "</dl>\n";
-	}
-	print $FH "<hr align='center'/>\n";
-	if (scalar @elements) {
-		print $FH "<h2><font color='#FF0000'>Element tree.</font></h2>\n";
-		print $FH "<ul>\n";
+	print $FH "<h2>Element tree.</h2>\n";
+	print $FH "<ul>\n";
+	if ($frameset) {
+		print $FH "  <li><a target='main' href='",$outfile,".main.html#elt_",$self->{root_name},"'><b>",$self->{root_name},"</b></a>\n";
+	} else {
 		print $FH "  <li><a href='#elt_",$self->{root_name},"'><b>",$self->{root_name},"</b></a>\n";
-		$self->_mk_tree($FH, $self->{root_name});
-		print $FH "  </li>\n";
-		print $FH "</ul>\n";
-		print $FH "<hr align='center'/>\n";
 	}
+	$self->_mk_tree($FH, $frameset, $outfile, $self->{root_name});
+	print $FH "  </li>\n";
+	print $FH "</ul>\n";
+}
+
+sub generateMain {
+	my $self = shift;
+	my ($FH, $flag_comment) = @_;
 
 	if (defined $self->{doctype_decl}) {
-		print $FH "<h2>Document entity</h2>\n";
-		print $FH "<pre>";
+		print $FH "<h3>Document entity</h3>\n";
+		print $FH "<p>";
 		if (defined $self->{xml_decl}) {
 			my $version = $self->{xml_decl}->{Version};
 			my $encoding = $self->{xml_decl}->{Encoding} || "";
@@ -356,86 +407,100 @@ sub printToFileHandle {
 			if (exists $self->{xml_decl}->{Standalone}) {
 				$standalone = ($self->{xml_decl}->{Standalone}) ? "yes" : "no";
 			}
-			print $FH "&lt;<font color='#0000A0'>?xml</font> ";
-			print $FH "<font color='#0000A0'>version</font>='<font color='#A00000'>",$version,"</font>' " if (defined $version);
-			print $FH "<font color='#0000A0'>encoding</font>='",$encoding,"' " if (defined $encoding);
-			print $FH "<font color='#0000A0'>standalone</font>='<font color='#A00000'>",$standalone,"</font>' " if ($standalone);
+			print $FH "&lt;<span class='keyword1'>?xml</span> ";
+			print $FH "<span class='keyword1'>version</span>='<span class='keyword2'>",$version,"</span>' "
+					if (defined $version);
+			print $FH "<span class='keyword1'>encoding</span>='",$encoding,"' "
+					if (defined $encoding);
+			print $FH "<span class='keyword1'>standalone</span>='<span class='keyword2'>",$standalone,"</span>' "
+					if ($standalone);
 			print $FH "?&gt;\n";
+			print $FH "<br />\n";
 		}
 		my $name = $self->{doctype_decl}->{Name};
-		print $FH "&lt;<font color='#0000A0'>!DOCTYPE</font> ",$name," [\n";
+		print $FH "&lt;<span class='keyword1'>!DOCTYPE</font> ",$name," [\n";
+		print $FH "<pre>\n";
 		print $FH "\t...\n";
 		print $FH "]&gt;\n";
 		print $FH "</pre>\n";
-		foreach my $comment (@{$self->{prolog_comments}}) {
-			my $data = $comment->{Data};
-			print $FH "comment $data\n";
+		print $FH "</p>\n";
+		if ($flag_comment) {
+			foreach my $comment (@{$self->{doctype_decl}->{comments}}) {
+				my $data = $self->_process_text($comment->{Data});
+				print $FH "    <p class='comment'>",$data,"</p>\n";
+			}
 		}
 	}
 
 	print $FH "<ul>\n";
 	foreach my $decl (@{$self->{list_decl}}) {
-		print $FH "  <li>\n";
 		my $type = $decl->{type};
 		my $name = $decl->{Name};
 		if      ($type eq "notation") {
 			my $publicId = $decl->{PublicId};
 			my $systemId = $decl->{SystemId};
-			print $FH "    <h3><a name='not_",$name,"'/>",$name,"</h3>\n";
-			print $FH "<pre>&lt;<font color='#0000A0'>!NOTATION</font> ",$name," ";
+			print $FH "  <li>\n";
+			print $FH "    <h3><a id='not_",$name,"' name='not_",$name,"'/>",$name,"</h3>\n";
+			print $FH "<p>&lt;<span class='keyword1'>!NOTATION</span> ",$name," ";
 			if      (defined $publicId and defined $systemId) {
-				print $FH "<font color='#0000A0'>PUBLIC</font> '",$publicId,"' '",$systemId,"'";
+				print $FH "<span class='keyword1'>PUBLIC</span> '",$publicId,"' '",$systemId,"'";
 			} elsif (defined $publicId) {
-				print $FH "<font color='#0000A0'>PUBLIC</font> '",$publicId,"'";
+				print $FH "<span class='keyword1'>PUBLIC</span> '",$publicId,"'";
 			} elsif (defined $systemId) {
-				print $FH "<font color='#0000A0'>SYSTEM</font> '",$systemId,"'";
+				print $FH "<span class='keyword1'>SYSTEM</span> '",$systemId,"'";
 			} else {
 				warn __PACKAGE__,":printToFileHandle INTERNAL_ERROR (NOTATION $name)\n";
 			}
-			print $FH " &gt;</pre>\n";
+			print $FH " &gt;</p>\n";
 		} elsif ($type eq "unparsed_entity") {
 			my $systemId = $decl->{SystemId};
 			my $publicId = $decl->{PublicId};
-			print $FH "    <h3><a name='ent_",$name,"'/>",$name,"</h3>\n";
-			print $FH "<pre>&lt;<font color='#0000A0'>!ENTITY</font> ",$name," ";
+			print $FH "  <li>\n";
+			print $FH "    <h3><a id='ent_",$name,"' name='ent_",$name,"'/>",$name,"</h3>\n";
+			print $FH "<p>&lt;<span class='keyword1'>!ENTITY</span> ",$name," ";
 			if (defined $publicId) {
-				print $FH "<font color='#0000A0'>PUBLIC</font> '",$publicId,"' '",$systemId,"'";
+				print $FH "<span class='keyword1'>PUBLIC</span> '",$publicId,"' '",$systemId,"'";
 			} else {
-				print $FH "<font color='#0000A0'>SYSTEM</font> '",$systemId,"'";
+				print $FH "<span class='keyword1'>SYSTEM</span> '",$systemId,"'";
 			}
-			print $FH " &gt;</pre>\n";
+			print $FH " &gt;</p>\n";
 		} elsif ($type eq "entity") {
 			my $value = $decl->{Value};
 			my $systemId = $decl->{SystemId};
 			my $publicId = $decl->{PublicId};
 			my $notation = $decl->{Notation};
-			print $FH "    <h3><a name='ent_",$name,"'/>",$name,"</h3>\n";
-			print $FH "<pre>&lt;<font color='#0000A0'>!ENTITY</font> ",$name," ";
+			print $FH "  <li>\n";
+			print $FH "    <h3><a id='ent_",$name,"' name='ent_",$name,"'/>",$name,"</h3>\n";
+			print $FH "<p>&lt;<span class='keyword1'>!ENTITY</span> ",$name," ";
 			if (defined $value) {
 				$value =~ s/&/&amp;/g;
 				print $FH "'",$value,"'";
 			} else {
 				if (defined $publicId) {
-					print $FH "<font color='#0000A0'>PUBLIC</font> '",$publicId,"' '",$systemId,"'";
+					print $FH "<span class='keyword1'>PUBLIC</span> '",$publicId,"' '",$systemId,"'";
 				} else {
-					print $FH "<font color='#0000A0'>SYSTEM</font> '",$systemId,"'";
+					print $FH "<span class='keyword1'>SYSTEM</span> '",$systemId,"'";
 				}
-				print $FH "<font color='#0000A0'>NDATA</font> <a href='#not_",$notation,"'>",$notation,"</a> "
+				print $FH "<span class='keyword1'>NDATA</span> <a href='#not_",$notation,"'>",$notation,"</a> "
 						 if (defined $notation);
 			}
-			print $FH " &gt;</pre>\n";
+			print $FH " &gt;</p>\n";
 		} elsif ($type eq "element") {
+			next unless (exists $self->{hash_element}->{$name});
 			my $model = $decl->{Model};
 			my $f_model = $self->_format_content_model($model);
-			print $FH "    <h3><a name='elt_",$name,"'/>",$name,"</h3>\n";
-			print $FH "<pre>&lt;<font color='#0000A0'>!ELEMENT</font> ",$name," ",$f_model," &gt;\n";
+			print $FH "  <li>\n";
+			print $FH "    <h3><a id='elt_",$name,"' name='elt_",$name,"'/>",$name,"</h3>\n";
+			print $FH "<p>&lt;<span class='keyword1'>!ELEMENT</span> ",$name," ",$f_model," &gt;\n";
 			if (exists $self->{hash_attr}->{$name}) {
 				foreach my $attr (@{$self->{hash_attr}->{$name}}) {
 					my $attr_name = $attr->{AttributeName};
 					my $type = $attr->{Type};
 					my $default = $attr->{Default};
 					my $fixed = $attr->{Fixed};
-					print $FH "&lt;<font color='#0000A0'>!ATTLIST</font> ",$name;
+#					print $FH "<br/>\n";		Bug with Netscape 4.7
+					print $FH "<br />\n";		# Fix
+					print $FH "&lt;<span class='keyword1'>!ATTLIST</span> ",$name;
 					print $FH " ",$attr_name;
 					if       ( $type eq "CDATA"
 							or $type eq "ID"
@@ -445,52 +510,53 @@ sub printToFileHandle {
 							or $type eq "ENTITIES"
 							or $type eq "NMTOKEN"
 							or $type eq "NMTOKENS" ) {
-						print $FH " <font color='#0000A0'>",$type,"</font>";
+						print $FH " <span class='keyword1'>",$type,"</span>";
 					} else {
 						print $FH " ",$type;
 					}
-					print $FH " <font color='#A00000'>",$default,"</font>";
-					print $FH " ",$fixed if (defined $fixed);
+					print $FH " <span class='keyword2'>#FIXED</span>" if (defined $fixed);
+					if ($default =~ /^#/) {
+						print $FH " <span class='keyword2'>",$default,"</span>";
+					} else {
+						print $FH " ",$default;
+					}
 					print $FH " &gt;\n";
 				}
 			}
-			print $FH "</pre>\n";
+			print $FH "</p>\n";
 		} else {
 			warn __PACKAGE__,":printToFileHandle INTERNAL_ERROR (type:$type)\n";
 		}
-		if (exists $decl->{comments}) {
-			foreach my $comment (@{$decl->{comments}}) {
-				my $data = $self->_process_text($comment->{Data});
-				print $FH "    <p><font color='#00A000'>\n";
-				print $FH "$data\n";
-				print $FH "    </font></p>\n";
-			}
-		}
-		if ($type eq "element" and exists $self->{hash_attr}->{$name}) {
-			print $FH "  <ul>\n";
-			foreach my $attr (@{$self->{hash_attr}->{$name}}) {
-				if (exists $attr->{comments}) {
-					my $attr_name = $attr->{AttributeName};
-					print $FH "    <li>",$attr_name," : <font color='#00A000'>\n";
-					my $first = 1;
-					foreach my $comment (@{$attr->{comments}}) {
-						my $data = $self->_process_text($comment->{Data});
-						print $FH "<p>\n" unless ($first);
-						print $FH "$data\n";
-						print $FH "</p>\n" unless ($first);
-						$first = 0;
-					}
-					print $FH "    </font></li>\n";
+		if ($flag_comment) {
+			if (exists $decl->{comments}) {
+				foreach my $comment (@{$decl->{comments}}) {
+					my $data = $self->_process_text($comment->{Data});
+					print $FH "    <p class='comment'>",$data,"</p>\n";
 				}
 			}
-			print $FH "  </ul>\n";
+			if ($type eq "element" and exists $self->{hash_attr}->{$name}) {
+				print $FH "  <ul>\n";
+				foreach my $attr (@{$self->{hash_attr}->{$name}}) {
+					if (exists $attr->{comments}) {
+						my $attr_name = $attr->{AttributeName};
+						print $FH "    <li>",$attr_name," : <span class='comment'>\n";
+						my $first = 1;
+						foreach my $comment (@{$attr->{comments}}) {
+							my $data = $self->_process_text($comment->{Data});
+							print $FH "<p>\n" unless ($first);
+							print $FH $data,"\n";
+							print $FH "</p>\n" unless ($first);
+							$first = 0;
+						}
+						print $FH "    </span></li>\n";
+					}
+				}
+				print $FH "  </ul>\n";
+			}
 		}
-		if ($type eq "element" and scalar @{$decl->{used_by}} != 0) {
-			my %done;
-			print $FH "  <p>Used by : ";
-			foreach (sort @{$decl->{used_by}}) {
-				next if (exists $done{$_});
-				$done{$_} = 1;
+		if ($type eq "element" and scalar keys %{$decl->{used_by}} != 0) {
+			print $FH "  <p>Child of : ";
+			foreach (sort keys %{$decl->{used_by}}) {
 				print $FH "<a href='#elt_",$_,"'>",$_,"</a> ";
 			}
 			print $FH "  </p>\n";
@@ -498,7 +564,88 @@ sub printToFileHandle {
 		print $FH "  </li>\n";
 	}
 	print $FH "</ul>\n";
-	$self->_format_tail($FH);
+}
+
+sub generateHTML {
+	my $self = shift;
+	my ($outfile, $frameset, $title, $flag_comment, $flag_multi, $flag_zombi) = @_;
+
+	$self->_cross_ref($flag_zombi);
+
+	$title = "DTD " . $self->{root_name}
+			unless (defined $title);
+
+	if ($flag_multi) {
+		foreach my $decl (@{$self->{list_decl}}) {
+			if (exists $decl->{comments}) {
+				$decl->{comments} = [ ${$decl->{comments}}[-1] ];
+			}
+		}
+	}
+
+	if (defined $frameset) {
+		open OUT, "> $outfile.html"
+				or die "can't open $outfile.html ($!)\n";
+		$self->_format_head(\*OUT, $title, 1);
+		print OUT "  <frameset cols='25%,75%'>\n";
+		print OUT "    <frameset rows='50%,50%'>\n";
+		print OUT "      <frame src='",$outfile,".alpha.html' id='alpha' name='alpha'/>\n";
+		print OUT "      <frame src='",$outfile,".tree.html' id='tree' name='tree'/>\n";
+		print OUT "    </frameset>\n";
+		print OUT "    <frame src='",$outfile,".main.html' id='main' name='main'/>\n";
+		print OUT "  </frameset>\n";
+		$self->_format_tail(\*OUT, 1);
+		close OUT;
+		open OUT, "> $outfile.alpha.html"
+				or die "can't open $outfile.alpha.html ($!)\n";
+		$self->_format_head(\*OUT, $title . " (Index)", 0);
+		print OUT "  <body>\n";
+		$self->generateAlpha(\*OUT, 1, $outfile);
+		print OUT "  </body>\n";
+		$self->_format_tail(\*OUT, 0);
+		close OUT;
+		open OUT, "> $outfile.tree.html"
+				or die "can't open $outfile.tree.html ($!)\n";
+		$self->_format_head(\*OUT, $title . " (Tree)", 0);
+		print OUT "  <body>\n";
+		$self->generateTree(\*OUT, 1, $outfile);
+		print OUT "  </body>\n";
+		$self->_format_tail(\*OUT, 0);
+		close OUT;
+		open OUT, "> $outfile.main.html"
+				or die "can't open $outfile.main.html ($!)\n";
+		$self->_format_head(\*OUT, $title . " (Main)", 0);
+		print OUT "  <body>\n";
+		print OUT "    <h1>",$title,"</h1>\n";
+		print OUT "    <hr />\n";
+		$self->generateMain(\*OUT, $flag_comment);
+		print OUT "    <hr />\n";
+		print OUT "    <cite>Generated by dtd2html</cite>\n";
+		print OUT "  </body>\n";
+		$self->_format_tail(\*OUT, 0);
+		close OUT;
+	} else {
+		open OUT, "> $outfile.html"
+				or die "can't open $outfile.html ($!)\n";
+		$self->_format_head(\*OUT, $title, 0);
+		print OUT "  <body>\n";
+		print OUT "    <h1>",$title,"</h1>\n";
+		print OUT "    <hr />\n";
+		$self->generateAlpha(\*OUT, 0);
+		print OUT "    <hr />\n";
+		if (scalar keys %{$self->{hash_element}}) {
+			$self->generateTree(\*OUT, 0);
+			print OUT "    <hr />\n";
+		}
+		$self->generateMain(\*OUT, $flag_comment);
+		print OUT "    <hr />\n";
+		print OUT "    <cite>Generated by dtd2html</cite>\n";
+		print OUT "\n";
+		print OUT "  </body>\n";
+		print OUT "\n";
+		$self->_format_tail(\*OUT, 0);
+		close OUT;
+	}
 }
 
 1;
@@ -519,6 +666,8 @@ XML::Handler::Dtd2Html - PerlSAX handler for generate a HTML documentation from
   $my_parser = new XML::Parser::PerlSAX(Handler => $my_handler, ParseParamEnt => 1);
   $result = $my_parser->parse( [OPTIONS] );
 
+  $result->generateHTML($outfile, $frameset, $title);
+
 =head1 DESCRIPTION
 
 All comments before a declaration are captured.
@@ -527,7 +676,7 @@ All entity references inside attribute values are expanded.
 
 =head1 AUTHOR
 
-Francois Perrad, perrad@besancon.sema.slb.com
+Francois Perrad, francois.perrad@gadz.org
 
 =head1 SEE ALSO
 
